/*
 * Additional functions used by the VTK visualiser
 * currently not used, since it is a pain to link on windows
 *
 */

#include "Python.h"
#include "Numeric/arrayobject.h"

#include "vtkPoints.h"
#include "vtkPythonUtil.h"

#include <malloc.h>

static PyObject * make_vtkpoints(PyObject *self, PyObject *args);

static PyMethodDef methods[] = {
  {"make_vtkpoints", make_vtkpoints, METH_VARARGS, "make vtkPoints structure"},
  {NULL            , NULL          , 0           , NULL                      }
};

PyMODINIT_FUNC initvtk_visualiser_ext(void){
  (void) Py_InitModule("vtk_visualiser_ext", methods);
  import_array();
}

/* Construct a vtkPoints object, given the number of triangles,
 * an array of vertex quantity values, an array of vertex x
 * and y coordinates and an array of triangle data.
 */
static PyObject * make_vtkpoints(PyObject *self, PyObject * args){
  int i, j;
  int n_tri;
  int n_vert;
  double scale;
  PyArrayObject * qty_values;
  PyArrayObject * vertex_values;
  PyArrayObject * triangles;
  double * qty_index;
  vtkPoints * grid;
  int dims[1];
  if(!PyArg_ParseTuple(args, "iidOOO", &n_tri, &n_vert, &scale, &qty_values, &vertex_values, &triangles))
    return NULL;
  
  dims[0] = n_vert;
  qty_index = (double*)malloc(n_vert * sizeof(double));
  if(qty_index == NULL)
    return NULL;
  
  for(i = 0 ; i < n_tri ; i++)
    for(j = 0 ; j < 3 ; j++)
      qty_index[*(int*)((triangles->data) + i*(triangles->strides[0]) + j*(triangles->strides[1]))] =
	*(double*)((qty_values->data) + i*(qty_values->strides[0]) + j*(qty_values->strides[1])) * scale;

  grid = vtkPoints::New();
  for(i = 0 ; i < n_vert ; i++)
    grid->InsertNextPoint(*(double*)((vertex_values->data) + i*(vertex_values->strides[0])),
			*(double*)((vertex_values->data) + i*(vertex_values->strides[0]) + (vertex_values->strides[1])),
			qty_index[i]);
  free(qty_index);
  return Py_BuildValue("O", vtkPythonGetObjectFromPointer(grid));
}
